
package org.dronus.graph;


import  java.util.*;

import org.dronus.graph.gui.*;

/**  GraphEltSet contains data about the graph's components. Currently
  *  the only components are edges and nodes.
  *
  * @author   Alexander Shapiro
  * @author   Murray Altheim (added support for IDs)
  * @version  1.21  $Id: GraphEltSet.java,v 1.24 2002/04/02 05:48:51 x_ander Exp $
  */
public class Graph implements ImmutableGraph {

    /*public Vector nodes= new Vector();
    public Vector edges= new Vector();
	*/
	protected HashSet nodes= new HashSet();
	protected HashSet edges= new HashSet();
	
	/*protected Set nodes= Collections.synchronizedSet(new HashSet());
	protected Set edges= Collections.synchronizedSet(new HashSet());
	*/

    /** The Hashtable containing references to the Node IDs of the current graph. */    
    protected Hashtable nodeIDRegistry = new Hashtable();
	// ...........

    /** Default constructor. */
    /*public GraphEltSet() {
        nodes = new Vector();
        edges = new Vector();
    }*/

  // Node manipulation ...........................

    /** Return the Node at int <tt>index</tt>, null if none are available. */
    /*public LayoutableNode nodeAt( int i ) {
        if ( nodes.size() == 0 ) return null;
        return (LayoutableNode)nodes.elementAt(i);
    }*/

    /** Return the number of Nodes in the cumulative Vector. */
    public int nodeCount() {
        return nodes.size(); 
    }

    /** Registers the Node <tt>node</tt> via its ID String <tt>id</tt>. 
      *
      * @param id the ID of the object.
      * @param node the Node to be registered.
      */
    //protected void registerNode( String id, Node node ) {
  // FIXME
    //}

    /** Add the Node <tt>node</tt> to the graph, and 
      * registers the Node via its ID. If no ID exists, no registration occurs. */
    public void addNode( LayoutableNode node ) throws GraphException {
        String id = node.getID();
        if ( id != null ) {
            if ( findNode(id) == null ) { // doesn't already exist
                nodeIDRegistry.put(id,node);				
                nodes.add(node);
            } else throw new GraphException(GraphException.NODE_EXISTS,"node ID '"+id+"' already exists.");
        } else {
            String label = node.getLabel().trim();
            if (label == null) label = "";
            if (!label.equals("") && findNode(node.getLabel()) == null ) {
                id = label;
            } else {
                int i;
                for( i = 1; findNode( label +"-"+ i ) != null; i++ );
                id = label + "-" + i; 
            } 
            node.setID(id);
            nodeIDRegistry.put(id,node);
            nodes.add(node);   
        }
        //} else throw new TGException(TGException.NODE_NO_ID,"node has no ID."); // could be ignored?
    }

    /** Returns true if the graph contains the Node <tt>node</tt>. */
    public boolean contains( LayoutableNode node ) {
        return nodes.contains(node);
    }

      /** Return the number of Edges in the cumulative Vector. */
    public int edgeCount() {
        return edges.size(); 
    }

    /** Add the Edge <tt>edge</tt> to the graph. */
    public void addEdge( LayoutableEdge edge ) {
        if ( edge == null ) return;
        if(!contains(edge)) {
            edges.add(edge);
            edge.from.addEdge(edge);
            edge.to.addEdge(edge);
        }
    }

    /** Returns true if the graph contains the Edge <tt>edge</tt>. */
    public boolean contains( LayoutableEdge edge ) {
        return edges.contains(edge);
    }

    /** Return the Node whose ID matches the String <tt>id</tt>, null if no match is found. */
    public LayoutableNode findNode( String id ) {
        if ( id == null ) return null; // ignore
        return (LayoutableNode)nodeIDRegistry.get(id);
    }

    /** Return an Edge spanning Node <tt>from</tt> to Node <tt>to</tt>. */
    public LayoutableEdge findEdge( LayoutableNode from, LayoutableNode to ) {
        for ( int i = 0 ; i < from.edgeCount(); i++ ) {
            LayoutableEdge e = from.edgeAt(i);
            if (e.to == to) return e;
        }
        return null;
    }

    /** Delete the Edge <tt>edge</tt>. */
    public boolean deleteEdge( LayoutableEdge edge ) {
        synchronized(edges) {
            if ( edge == null ) return false;
            if (!edges.remove(edge)) return false;
            edge.from.removeEdge(edge);
            edge.to.removeEdge(edge);
            return true;
        }
    }

    /** Delete the Node <tt>node</tt>, returning true if successful. */
    synchronized public boolean deleteNode( LayoutableNode node ) {
        if ( node == null ) return false;
        if ( !nodes.remove(node) ) return false;
    
        String id = node.getID();
        if ( id != null ) nodeIDRegistry.remove(id); // remove from registry
        for ( int i = 0 ; i < node.edgeCount(); i++ ) {
            LayoutableEdge e = node.edgeAt(i);
            if ( e.from == node ) {
                edges.remove(e); // Delete edge not used, because it would change the node's edges
                e.to.removeEdge(e);     // vector which is being iterated on.
            } else if ( e.to == node ) {
                edges.remove(e);
                e.from.removeEdge(e);
            }
            //No edges are deleted from node.  Hopefully garbage collection will pick them up.
        }
         return true;
    }

    /** Clear all nodes and edges. */
	synchronized public void clearAll() {
		nodes.clear();
		edges.clear();
		nodeIDRegistry.clear();
    }

   /** A way of iterating through all the nodes.
     * Maybe too complex, and should be replaced by iterators.
     */
   synchronized public void forAllNodes( GraphForEachNode fen ) {
    	Iterator i=nodes.iterator(); 
    	while(i.hasNext()){
    	    LayoutableNode n = (LayoutableNode)i.next();
            fen.forEachNode(n);
       }
    }

    /** iterates through pairs of Nodes. */
	synchronized public void forAllNodePairs( GraphForEachNodePair fenp ) {
    	LayoutableNode[] ns=(LayoutableNode[])nodes.toArray(new LayoutableNode[0]);
        for (int i=0;i<nodeCount();i++) {
            LayoutableNode n1=ns[i];
            fenp.beforeInnerLoop(n1);            
            for (int j=i+1;j<nodeCount();j++)
                fenp.forEachNodePair(n1, ns[j]);            
            fenp.afterInnerLoop(n1);
        }
    }
	
	

    /** Iterates through Edges. */
    synchronized public void forAllEdges( GraphForEachEdge fee ) {
    	Iterator i=edges.iterator();
        while(i.hasNext())
            fee.forEachEdge((LayoutableEdge)i.next());
    }
} 
