
package org.dronus.graph;

import  java.util.*;

import org.dronus.graph.gui.*;
import org.dronus.graph.nodefilter.*;

/** GraphUtils is a set of functions that return information about a Graph 
  *
  * @author   Alexander Shapiro
  * @version  1.21  $Id: GraphUtils.java,v 1.2 2002/04/01 05:55:45 x_ander Exp $
  */
public class GraphUtils {
   
    public static Hashtable calculateDistances(Graph ges, LayoutableNode focusNode, int radius, 
                                               int maxAddEdgeCount, int maxExpandEdgeCount, 
                                               boolean unidirectional, NodeFilter filter ) {
        Hashtable distHash = new Hashtable();
        distHash.put(focusNode,new Integer(0));

        GraphNodeQueue nodeQ = new GraphNodeQueue();
        nodeQ.push(focusNode);

        while (!nodeQ.isEmpty()) {
            LayoutableNode n = (LayoutableNode)nodeQ.pop();
            int currDist = ((Integer) distHash.get(n)).intValue();

            if (currDist>=radius) break;

            for (int i = 0 ; i < n.edgeCount(); i++) {
                LayoutableEdge e=n.edgeAt(i);
                if(n!=n.edgeAt(i).from && unidirectional) continue;
                LayoutableNode adjNode = e.getOtherEndpt(n);
                if(ges.contains(e) && !distHash.containsKey(adjNode) && adjNode.edgeCount()<=maxAddEdgeCount) {
					if (filter==null || !filter.isCutting() || filter.match(adjNode)){
                		if (adjNode.edgeCount()<=maxExpandEdgeCount) nodeQ.push(adjNode);                	
                     	distHash.put(adjNode,new Integer(currDist+1));
                	}
                }
            }
        }
        return distHash;
    }

    public static Hashtable calculateDistances(Graph ges, LayoutableNode focusNode, int radius ) {        
        return calculateDistances(ges, focusNode, radius, 1000, 1000, false, null);
    }

   /** A temporary function that returns the largest connected subgraph in a GraphEltSet.  
     */
    
    public static Collection getLargestConnectedSubgraph(Graph ges) {
        int nodeCount = ges.nodeCount();
        if(nodeCount==0) return null;
        
        Vector subgraphVector = new Vector();
        Iterator i=ges.nodes.iterator();
        while(i.hasNext()){
            LayoutableNode n = (LayoutableNode)i.next();
            boolean skipNode=false;
            for (int j = 0; j<subgraphVector.size();j++) {
                if(((Collection) subgraphVector.elementAt(j)).contains(n)) {
                    skipNode=true;
                }
            }   
            Collection subgraph = calculateDistances(ges,n,1000).keySet();                     
            if(subgraph.size()>nodeCount/2) return subgraph; //We are done
            if (!skipNode) subgraphVector.add(subgraph);  
            
        }
        
        int maxSize=0;
        int maxIndex=0;
        for (int j = 0; j<subgraphVector.size();j++) {
            if(((Collection) subgraphVector.elementAt(j)).size()>maxSize) {
                maxSize = ((Collection) subgraphVector.elementAt(j)).size();
                maxIndex = j;
            }
        }   
       
        return (Collection) subgraphVector.elementAt(maxIndex);
    }

} // end com.touchgraph.graphlayout.graphelements.GraphEltSet
