package org.dronus.graph.gui;


/*
 * der Layout-Task, der den Grafen auslegt.
 * wichtigste zwei Funktionen sind relaxEdges und avoidLabels
 * */


public class GraphLayout implements Runnable {
	
    //private ImmutableGraphEltSet graphEltSet;
    private GraphPanel tgPanel;
    private Thread relaxer;
    private double damper=0.0;      
    private double maxMotion=0;     
    private double lastMaxMotion=0;
    private double motionRatio = 0; // It's sort of a ratio, equal to lastMaxMotion/maxMotion-1
    
    
	int repellCount;
    
	double sumEdgeLen=0;
    
    LayoutableNode dragNode=null;
    
  // ............

  /** Constructor with a supplied TGPanel <tt>tgp</tt>.
    */
    public GraphLayout( GraphPanel tgp ) {
        tgPanel = tgp;
        relaxer = null;
    }


    void setDragNode(LayoutableNode n) {
        dragNode = n;
    }


	/* sorgt dafr, dass die Kanten sich spannen, um den Grafen zusammenziehen 
	 * der name relax ist bullshit.
	 */
    private synchronized void relaxEdges() {
    	//es wird eine anonyme Klasse benutzt, um die Funktion auf alle
    	//Kanten zu "mappen". Bisserl seltsame architektur, aber flexibel.
    	final GraphForEachEdge fee = new GraphForEachEdge() {
    		//diese Funktion wird also fr alle Kanten ausgefhrt!
			  public void forEachEdge(LayoutableEdge e) {				
                //x,y distanz der zwei Knoten
                double dx = e.to.x - e.from.x;
                double dy = e.to.y - e.from.y;
                //Euklid
				double len = Math.sqrt(dx * dx + dy * dy)/1000.0;
                //double len = Math.sqrt(dx * dx + dy * dy)/500.0;
//				double len = (Math.sqrt(dx * dx + dy * dy)-100)/500.0;
//				sumEdgeLen+=len;

   	            /*nun wirds etwas "spanisch", die Lngen
   	             * werden nach der Belastung der Knoten mit Nachbarn gewichtet.
   	             * 
   	             * das geht auch ohne:*/
   	              //double lenTo = len; double lenFrom = len;
   	             /* was bei balancierten graphen auch ok ist. 
   	             *
   	            
				
				der vorteil von der gewichtung ist: "beliebte" knoten lassen platz fr
				ihre nachbarn. das ist dann gut, wenn der graph stark unregulr ist
				(anzahl der nachbarknoten variiert stark) 
			
				*/
				/*double lenTo = len / (e.to.visibleEdgeCnt);
				double lenFrom = len / (e.from.visibleEdgeCnt);
				*/
				double lenTo = len / (e.to.visibleEdgeCnt+e.from.visibleEdgeCnt-1);
				double lenFrom = lenTo;
//				if (lenTo<1) System.out.println("AUA!!!!"); 
				
				/*
		 		oder mit einem offset:  
   	            double lenTo = len / (e.to.visibleEdgeCnt+2);
				double lenFrom = len / (e.from.visibleEdgeCnt+2);
				besser, wenn man sehr wenig platz hat. sonst ist obiges besser, da es
				"beliebten" knoten ermglicht, sich mit ihren nachbarn ein groes 
				gebiet zu besetzten. mit diesem hier wird der graph kompakter, dafr unbersichtlicher.
				 */
                
                //und die gewnschten bewegungen einspeichern.
                //die knoten haben einen geschwindikeits-vektor dafr: dx,dy.
                e.to.dx -= dx*lenTo;
                e.to.dy -= dy*lenTo;
                e.from.dx += dx*lenFrom;
                e.from.dy += dy*lenFrom;
            }
        };
//		sumEdgeLen=0;
        tgPanel.getVisibleLocality().forAllEdges(fee);
//     	System.out.println(sumEdgeLen/tgPanel.getVisibleLocality().edgeCount());
     
    }

	/* sorgt dafr, dass die Knoten sich abstossen, um sich nicht zu verdecken
	 * und dem Graphen aufzuspannen
	 * 
	 */
    private synchronized void avoidLabels() {
		// die reichweite ab der die Abstossungsfunktion auf Null gesetzt wird (eine Nherung...)
		//final double RANGE=250;				
		final double RANGE=400;
		final double RANGE2=RANGE*RANGE;		
		//statistik
		//repellCount=0;
		//die Knoten-Paar-Funktionsklasse
        final GraphForEachNodePair fenp = new GraphForEachNodePair() {
        	//folgende Funktion wird dann spter fr alle Knotenpaare unterhalb dist. 250 ausgefhrt
        	//(das hat nichts mit der Kantenpaarung zu tun.)
            final public void forEachNodePair(LayoutableNode n1, LayoutableNode n2) {
				//knoten-distanzen dx,dy
		        double dx = n1.x - n2.x, dy = n1.y - n2.y; 
                //pr-euklid (ohne wurzel)
                double len = dx * dx + dy * dy;
				//strke der abstossung (=gre des grafen)
				final double repSum= 250, kickSum=1;
				//final double repSum= 100, kickSum=1;
				//singularitt: knoten genau aufeinander?
                if (len < .00001) {
                    //unmotiviert auseinanderbringen
                    n1.dx = n2.dy+kickSum;  n1.dy = -n2.dx+kickSum;                    
                   	return;                    
				//knoten innerhalb der reichweite?
				//dieser test ist eigentlich bullshit, weil das ja die 
				//vorgeschaltete utility-f forNearNodePairs gewhrleistet
				//nur aus kompatibilittsgrnden zum voll-paar-iterator
                } else if (len <RANGE2) {
                	//abstossung ist differenz/(lnge^2 / strke)
                	// len ist _ohne_ wurzel s.o.!!!
                	// daher ist dx/=len etc. wirklich eine abfallende abstossung!
                	//(wre len gewurzelt, htten wir nur normiert!)
                	len/=repSum;
                	dx /=len; dy /=len;
                    //dx = dx / len *repSum; dy = dy / len *repSum; 
                } else return;              
                //und zu den knoten-geschwindikeiten zufgen.  
                n1.dx += dx;
                n1.dy += dy;
                n2.dx -= dx;
                n2.dy -= dy;
                //statistik bei bedarf
		//		  repellCount++;     
            }
        };
 
		//nun endlich die wunderbare util-f "forNearNodePairs". 
        tgPanel.getVisibleLocality().forNearNodePairs(fenp,RANGE);
		//tgPanel.getVisibleLocality().forAllNodePairs(fenp);
         
//		statistik bei bedarf     	
		//int nc=tgPanel.getVisibleLocality().nodeCount();	System.out.println(" made "+repellCount+" pairs out of possible "+(nc*nc/2));
    }

    public void resetDamper() {     damper = 1;  }

    //die fumktion die alle relaxer-anteile zusammenbringt
    /*man beachte mathematisch:
     * 
     * dies ist ein optimierungsaalgorithmus, und zwar ein recht einfacher 
     * ein sog. "trger gradientenabstieg" mit etwas schrittweitensteuerung
     * */
    private synchronized void moveNodes() {
    	
    	//eine messung der schnellsten bewegung fr die schrittweitensteuerung 
        lastMaxMotion = maxMotion;
        maxMotion=0;
        GraphForEachNode fen = new GraphForEachNode() {
            //wird fr jeden knoten ausgefhrt
            public void forEachNode(LayoutableNode n) {
                //differenzvektor
                double dx = n.dx, dy = n.dy;
                //eine zentrums-rckstellkraft, damit isolierte knoten/komponenten
                //sich nicht ins all aufmachen
                
                final double CENTER=0.002;
                dx-=n.x*CENTER;
				dy-=n.y*CENTER;
                
                
                //damper ist eine versteckte schrittweitensteuerung
                dx*=damper; dy*=damper;	
				//mittelung der trgheit 
				n.dx=dx/2.0;n.dy=dy/2.0;

                //wie weit will dieser Knoten gehen?
                double distMoved = Math.sqrt(dx*dx+dy*dy); 
				//ist der Knoten auch nicht angepinnt oder gerade am Mousepfeil hngend?
                 if (!n.fixed && !(n==dragNode) ) {
                 	//Sicherheit und Stabilitt durch sichere Grenzen - CDU                    
                 	n.x += Math.max(-150, Math.min(150, dx)); 
                    n.y += Math.max(-150, Math.min(150, dy));
                 }
                 //und die hchstgeschwindigkeit updaten
                 maxMotion=Math.max(distMoved,maxMotion);
            }
        };
		
		//und das ganze obige auf alle knoten anwenden.	
        tgPanel.getVisibleLocality().forAllNodes(fen);

        /* nun zur schrittweitensteuerung. 
         * hier ist historisch bedingt viel blut vergossen worden... 
         * numerische oprimierung ist eben was fr ingenieure
         */ 
       //wenn sich noch was bewegt, verhltnis der letzten zwei schritte ansehen.
       // es ndert sich nur wenig, dann muss irgendwann mal gedft werden.
/*	   if(motionRatio<=0.01 && motionRatio>-0.01) {    
		   //entweder es geht schon langsam, dann muss man irgendwann schluss
		   //machen, damit es nicht ewig rumwackelt. geht es zu schnell, und man hat bereits
		   //gedmpft, dann muss man dennoch dmpfen, da mit der groen schrittweite
		   //sonst stabile oder sogar explodierende oszilation eintritt!!! 
		   if ((maxMotion<1 || (maxMotion>10 && damper<0.9)) && damper > 0.1) damper -= 0.1;
		   else if(damper>0.01) damper -=0.01;
	   }
	   */
	   //wenn es schn ruhig ist, ganz stoppen, spart CPU.
		
		if (maxMotion>0) motionRatio = lastMaxMotion/maxMotion-1; 
		   else motionRatio = 0;                                     	   
		
		if (maxMotion<5 && motionRatio<0)damper-=.01;
		if (maxMotion<2 && motionRatio<0) damper-=.1;
		if(maxMotion<0.01)  damper=0;
		
		if(damper<0) damper=0;	   
	   	
	   
    }

	//einen Relax-Schritt machen.
    public synchronized void relax() {
       //do{       
          relaxEdges(); //Kanten spannen
          avoidLabels();//Knoten entspannen
          moveNodes();  //bewegen&dmpfer steuern.
  //        System.out.println(" damp " + damper + "  maxM " + maxMotion + "  motR " + motionRatio );
       //}while (maxMotion>200);
    }
    
    //eine Schleife, bis alles ruhig ist (sofern das in 1000 schritten klappt)
	public synchronized void relaxTotal() {
		
	   int i=0;
	   resetDamper();
	   do{       
		  relax();
			// System.out.print("RelaxStep:"+i);
	   }while (!(damper<0.1 && maxMotion<0.5) && i++<1000);
	   tgPanel.repaintAfterMove();
	}

	//-------------ab hier mickey-mouse-bullshit
    private void myWait() { //I think it was Netscape that caused me not to use Wait, or was it java 1.1?
 		try {
                Thread.sleep(10);
        } catch (InterruptedException e) {
        }
    }

    public void run() {
        Thread me = Thread.currentThread();
		me.setPriority(1);
        while (relaxer == me) {
            relax();
			tgPanel.repaintAfterMove();
            while(damper<0.1 && maxMotion<0.5) myWait();
        }
    }

    public void start() { 	relaxer = new Thread(this);relaxer.start();  }
    public void stop() { relaxer=null; }

} // end com.touchgraph.graphlayout.TGLayout
