
package org.dronus.graph.gui;

import org.dronus.graph.Graph;
import org.dronus.graph.VisibleLocality;
import  org.dronus.graph.gui.interact.*;
import org.dronus.graph.nodefilter.*;

import  java.awt.*;
import  java.awt.event.*;
import  java.util.*;

public abstract class GraphPanel extends Panel {
  
    public PopupMenu backPopup;

	public static Color BACK_COLOR = Color.BLACK;

    protected Graph completeEltSet;
    protected VisibleLocality visibleLocality;
    protected LocalityUtils localityUtils;
    public GraphLayout tgLayout = new GraphLayout(this);
    protected BasicMouseMotionListener basicMML;

    protected LayoutableEdge mouseOverE;  //mouseOverE is the edge the mouse is over
    protected LayoutableNode mouseOverN;  //mouseOverN is the node the mouse is over
    protected boolean maintainMouseOver = false; //If true, then don't change mouseOverN or mouseOverE
	protected NodeFilter filter=null;

    protected LayoutableNode select;
	protected int locality;

	
	protected LayoutableNode dragNode; //Node currently being dragged

    protected Point mousePos; //Mouse location, updated in the mouseMotionListener
	protected Image offscreen;
	protected boolean painting=false;
	protected Dimension offscreensize;
	protected Graphics offgraphics;

    private Vector graphListeners;
    private Vector paintListeners;
	protected LensSet tgLensSet=new LensSet();  // Converts between a nodes visual position (drawx, drawy),
                          // and its absolute position (x,y).
	protected AdjustOriginLens adjustOriginLens;
	protected SwitchSelectUI switchSelectUI;

	boolean antialias=false;
	
	public HVScroll hvScroll = new HVScroll(this, tgLensSet);
	
	// ............

   /** Default constructor.
     */
    public GraphPanel()
    {
        setLayout(null);
		setUpBackPopup();
		completeEltSet = new Graph();
		visibleLocality = new VisibleLocality(completeEltSet);
		localityUtils = new LocalityUtils(visibleLocality, this);
				
        addMouseListener(new BasicMouseListener());
        basicMML = new BasicMouseMotionListener();
        addMouseMotionListener(basicMML);

        graphListeners=new Vector();
        paintListeners=new Vector();

        adjustOriginLens = new AdjustOriginLens();
		
		tgLensSet.addLens(hvScroll.getLens());
		//tgLensSet.addLens(adjustOriginLens);
		
        switchSelectUI = new SwitchSelectUI();
        
//        tgLayout.start();
    }

	public HVScroll getHVScroll()    {   return hvScroll;  }
	public void centerNode(LayoutableNode n){
		  hvScroll.slowScrollToCenter(n);
   }
      
/*    public void setLensSet( TGLensSet lensSet ) {
        tgLensSet = lensSet;
    }*/
    /*public AdjustOriginLens getAdjustOriginLens() {
        return adjustOriginLens;
    }
*/
    public VisibleLocality getVisibleLocality() {
        return visibleLocality;
    }

    public LayoutableNode findNode( String id ) {
        if ( id == null ) return null; // ignore
        return completeEltSet.findNode(id);
    }
    
	abstract public LayoutableNode addNode();
  
    /** Add the Node <tt>node</tt> to the visibleLocality, checking for ID uniqueness. */
    public void addNode( final LayoutableNode node ){
        synchronized (localityUtils) {
            visibleLocality.addNode(node);
            resetDamper();                        
        }
    }

	public boolean deleteNode( LayoutableNode node ) {
        synchronized (localityUtils) {
            if (visibleLocality.deleteNode(node)) { // delete from visibleLocality, *AND completeEltSet
                if ( node == select ) clearSelect();
                resetDamper();
                return true;
            }
            return false;
        }
    }

    public void clearAll() {
        synchronized (localityUtils) {
            visibleLocality.clearAll();
        }
    }

    public LayoutableNode getSelect() {
        return select;
    }

    public LayoutableNode getMouseOverN() {
        return mouseOverN;
    }

    public synchronized void setMouseOverN( LayoutableNode node ) {
        if ( dragNode != null || maintainMouseOver ) return;  // So you don't accidentally switch nodes while dragging
        if ( mouseOverN != node ) {
            mouseOverN=node;
        }
    }

   public void deleteEdge( LayoutableEdge edge ) {
        synchronized (localityUtils) {
            visibleLocality.deleteEdge(edge);
            resetDamper();
        }
    }

    public LayoutableEdge findEdge( LayoutableNode f, LayoutableNode t ) {
        return visibleLocality.findEdge(f, t);
    }

    public void addEdge(LayoutableEdge e) {
        synchronized (localityUtils) {
            visibleLocality.addEdge(e); resetDamper();
        }
    }

    public LayoutableEdge getMouseOverE() {  return mouseOverE; }
    public synchronized void setMouseOverE( LayoutableEdge edge ) {
        if ( dragNode != null || maintainMouseOver ) return; // No funny business while dragging
        if ( mouseOverE != edge ) {
            mouseOverE = edge;
        }
    }

    protected class AdjustOriginLens extends AbstractLens {
        protected void applyLens(Point2D p) {
            p.x=p.x+GraphPanel.this.getSize().width/2;
            p.y=p.y+GraphPanel.this.getSize().height/2;
        }

        protected void undoLens(Point2D p) {
            p.x=p.x-GraphPanel.this.getSize().width/2;
            p.y=p.y-GraphPanel.this.getSize().height/2;
        }
    }

    public class SwitchSelectUI extends GraphAbstractClickUI {
        public void mouseClicked(MouseEvent e) {
            if ( mouseOverN != null ) {
                if ( mouseOverN != select )
                    setSelect(mouseOverN);
                else
                    clearSelect();
            }
        }
    }

  

    public synchronized void addPaintListener(GraphPaintListener pl){
        paintListeners.addElement(pl);
    } 
    public synchronized void removePaintListener(GraphPaintListener pl){
        paintListeners.removeElement(pl);
    }
    public void setMaintainMouseOver( boolean maintain ) {  maintainMouseOver = maintain; }

    public void clearSelect() {
        if ( select != null ) {
            select = null;
            repaint();
        }
    }
   
    public void setSelect( LayoutableNode node ) {
          if ( node != null ) {
              select = node;
              repaint();
          } else if ( node == null ) clearSelect();
    }

    public void setLocale( LayoutableNode node, int radius ){
    	  locality=radius;
    	  localityUtils.setLocale(node,radius);
    }
    public void expandNode( LayoutableNode node ) {
    	localityUtils.expandNode(node);
    }
    public void hideNode( LayoutableNode hideNode ) {
    	visibleLocality.removeNode(hideNode); 
    	//localityUtils.hideNode(hideNode );
    }
    public void collapseNode( LayoutableNode collapseNode ) {localityUtils.collapseNode(collapseNode );}
    public void hideEdge( LayoutableEdge hideEdge ) {
        visibleLocality.removeEdge(hideEdge);
        if ( mouseOverE == hideEdge ) setMouseOverE(null);
        resetDamper();
    }
	public void hideAll(){
		visibleLocality.removeAll();
	}
	
	public void applyFilter(){
		if (filter!=null) {
			filter.applyFilter(visibleLocality);
			//resetDamper();
			if (select!=null && !visibleLocality.contains(select)) clearSelect();
		} 
	}
	
    public void setDragNode( LayoutableNode node ) { dragNode = node; tgLayout.setDragNode(node);}
    public LayoutableNode getDragNode() { return dragNode; }
    public Point getMousePos() {  return mousePos;}
	public void start() {if (tgLayout!=null) tgLayout.start();}
	public void stop() { if (tgLayout!=null) tgLayout.stop();}
    
    public void resetDamper() {  if (tgLayout!=null) tgLayout.resetDamper();}
    
    class BasicMouseListener extends MouseAdapter {

        public void mouseEntered(MouseEvent e) {
            addMouseMotionListener(basicMML);
        }

        public void mouseExited(MouseEvent e) {
            removeMouseMotionListener(basicMML);
            mousePos = null;
            setMouseOverN(null);
            setMouseOverE(null);
            repaint();
        }
    }

    class BasicMouseMotionListener implements MouseMotionListener {
        public void mouseDragged(MouseEvent e) {
            mousePos = e.getPoint();
            findMouseOver();
            /*try {
                Thread.sleep(6); //An attempt to make the cursor flicker less
            } catch (InterruptedException ex) {
            }*/
        }

        public void mouseMoved( MouseEvent e ) {
            mousePos = e.getPoint();
            synchronized (this) {
                LayoutableEdge oldMouseOverE = mouseOverE;
                LayoutableNode oldMouseOverN = mouseOverN;
                findMouseOver();
                if (oldMouseOverE!=mouseOverE || oldMouseOverN!=mouseOverN) {
                    repaint();
                }           
            }
        }
    }

    protected synchronized void findMouseOver() {

        if ( mousePos == null ) {
            setMouseOverN(null);
            setMouseOverE(null);
            return;
        }

        final int mpx=mousePos.x;
        final int mpy=mousePos.y;

        final LayoutableNode[] monA = new LayoutableNode[1];
        final LayoutableEdge[] moeA = new LayoutableEdge[1];

        GraphForEachNode fen = new GraphForEachNode() {

            double minoverdist = 100; //Kind of a hack (see second if statement)
                                        //Nodes can be as wide as 200 (=2*100)
            public void forEachNode( LayoutableNode node ) {
                double x = node.drawx;
                double y = node.drawy;

                double dist = Math.sqrt((mpx-x)*(mpx-x)+(mpy-y)*(mpy-y));

                  if ( ( dist < minoverdist ) && node.containsPoint(mpx,mpy) ) {
                      minoverdist = dist;
                      monA[0] = node;
                  }
            }
        };
        visibleLocality.forAllNodes(fen);

        GraphForEachEdge fee = new GraphForEachEdge() {

            double minDist = 8; // Tangential distance to the edge
            double minFromDist = 1000; // Distance to the edge's "from" node

            public void forEachEdge( LayoutableEdge edge ) {
                double x = edge.from.drawx;
                double y = edge.from.drawy;
                double dist = edge.distFromPoint(mpx,mpy);
                if ( dist < minDist ) { // Set the over edge to the edge with the minimun tangential distance
                    minDist = dist;
                    minFromDist = Math.sqrt((mpx-x)*(mpx-x)+(mpy-y)*(mpy-y));
                    moeA[0] = edge;
                } else if ( dist == minDist ) { // If tangential distances are identical, chose
                                                // the edge whose "from" node is closest.
                    double fromDist = Math.sqrt((mpx-x)*(mpx-x)+(mpy-y)*(mpy-y));
                    if ( fromDist < minFromDist ) {
                        minFromDist = fromDist;
                        moeA[0] = edge;
                    }
                }
            }
        };
        visibleLocality.forAllEdges(fee);

        setMouseOverN(monA[0]);
        if ( monA[0] == null )
            setMouseOverE(moeA[0]);
        else
            setMouseOverE(null);
    }

    Point2D topLeftDraw = null;
    Point2D bottomRightDraw = null;

    public Point2D getTopLeftDraw() {
        return new Point2D(topLeftDraw);
    }

    public Point2D getBottomRightDraw() {
        return new Point2D(bottomRightDraw);
    }

    public Point2D getCenter() {
        return tgLensSet.convDrawToReal(getSize().width/2,getSize().height/2);
    }
    public Point2D getDrawCenter() {
        return new Point2D(getSize().width/2,getSize().height/2);
    }

    public void updateGraphSize() {
        if ( topLeftDraw == null ) topLeftDraw=new Point2D(0,0);
        if ( bottomRightDraw == null ) bottomRightDraw=new Point2D(0,0);

        GraphForEachNode fen = new GraphForEachNode() {
            boolean firstNode=true;
            public void forEachNode( LayoutableNode node ) {
                if ( firstNode ) { //initialize topRight + bottomLeft
                    topLeftDraw.x=node.drawx;
					topLeftDraw.y=node.drawy;
                    bottomRightDraw.x=node.drawx;
					bottomRightDraw.y=node.drawy;
                    firstNode=false;
                } else {  //Standard max and min finding
                    topLeftDraw.x=Math.min(node.drawx,topLeftDraw.x);
					topLeftDraw.y=Math.min(node.drawy,topLeftDraw.y);
                    bottomRightDraw.x=Math.max(node.drawx, bottomRightDraw.x);
					bottomRightDraw.y=Math.max(node.drawy, bottomRightDraw.y);
                }
            }
        };
        visibleLocality.forAllNodes(fen);
    }

    public synchronized void processGraphMove() {
        updateDrawPositions();
        updateGraphSize();
    }

    public synchronized void repaintAfterMove() { // Called by TGLayout + others to indicate that graph has moved
        processGraphMove();
        findMouseOver();
        //repaint();
        if (getGraphics()!=null) update(getGraphics());
    }

    public void updateDrawPos( LayoutableNode node ) { //sets the visual position from the real position
        Point2D p = tgLensSet.convRealToDraw(node.x,node.y);
        node.drawx = p.x;
        node.drawy = p.y;
       }

    public void updatePosFromDraw( LayoutableNode node ) { //sets the real position from the visual position
        Point2D p = tgLensSet.convDrawToReal(node.drawx,node.drawy);
        node.x = p.x;
        node.y = p.y;
    }

    public void updateDrawPositions() {
        GraphForEachNode fen = new GraphForEachNode() {
            public void forEachNode( LayoutableNode node ) {
                updateDrawPos(node);
            }
        };
        visibleLocality.forAllNodes(fen);
    }

    public void paint( Graphics g ) {
        if (!painting) {
        	painting=true;
        	update(g);
			painting=false;
        } 
    }

	public void setAntialias(boolean a){
		if (offgraphics==null) return;
		((Graphics2D)offgraphics).setRenderingHint(RenderingHints.KEY_ANTIALIASING, (a ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF));		
		((Graphics2D)offgraphics).setRenderingHint(RenderingHints.KEY_RENDERING, (a ? RenderingHints.VALUE_RENDER_QUALITY : RenderingHints.VALUE_RENDER_SPEED));
		LayoutableEdge.setAntialias(a);
		LayoutableNode.setAntialias(a);
		((Graphics2D)offgraphics).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		antialias=a;
	}

    public synchronized void update( Graphics g ) {  
        Dimension d = getSize();
        if ( (offscreen == null) 
                || ( d.width != offscreensize.width ) 
                || ( d.height != offscreensize.height )) {
            if (d.height<=0) d.height=100;
            offscreen = createImage(d.width, d.height);
            offscreensize = d;
            offgraphics = offscreen.getGraphics();
			setAntialias(antialias);						
            processGraphMove();            
            findMouseOver();            
        }

		offgraphics.setColor(BACK_COLOR);
        offgraphics.fillRect(0, 0, d.width, d.height);
		
        synchronized(this) {
            paintListeners = (Vector)paintListeners.clone();
        }

        for ( int i = 0; i < paintListeners.size(); i++ ) {
            GraphPaintListener pl = (GraphPaintListener) paintListeners.elementAt(i);
            pl.paintFirst(offgraphics);
        }

        GraphForEachEdge fee = new GraphForEachEdge() {
            public void forEachEdge( LayoutableEdge edge ) {
				if (edge.intersects(getSize()))  
					edge.paint(offgraphics, false, false);
            }
        };

        visibleLocality.forAllEdges(fee);

        for ( int i = 0; i < paintListeners.size() ; i++ ) {
             GraphPaintListener pl = (GraphPaintListener) paintListeners.elementAt(i);
             pl.paintAfterEdges(offgraphics);
        }

        GraphForEachNode fen = new GraphForEachNode() {
            public void forEachNode( LayoutableNode node ) {
				if (node.intersects(getSize()) )  
					node.paint(offgraphics,node==select,false);
            }
        };

        visibleLocality.forAllNodes(fen);

        if ( mouseOverE != null ) { //Make the edge the mouse is over appear on top.
            mouseOverE.paint(offgraphics, true, false);			
            mouseOverE.from.paint(offgraphics, mouseOverE.from==select,false);
            mouseOverE.to.paint(offgraphics, mouseOverE.to==select,false);
        }

        if ( select != null ) { //Make the selected node appear on top.
            select.paint(offgraphics, true,false);
        }

        if ( mouseOverN != null ) { //Make the node the mouse is over appear on top.
            mouseOverN.paint(offgraphics, mouseOverN==select,false);
        }

        for ( int i = 0; i < paintListeners.size(); i++ ) {
            GraphPaintListener pl = (GraphPaintListener)paintListeners.elementAt(i);
            pl.paintLast(offgraphics);
        }
        
        paintComponents(offgraphics); //Paint any components that have been added to this pane
        g.drawImage(offscreen, 0, 0, null);
    
    }
	public NodeFilter getFilter() {
		return filter;
	}

	public void setFilter(NodeFilter filter) {
		this.filter = filter;
		if (select!= null && !filter.match(select)) clearSelect();
		if (select!=null) 
			setLocale(select, locality);				
		else{
			//visibleLocality.fill();
			//applyFilter(visibleLocality());
			filter.applyFilter(visibleLocality);
			resetDamper(); repaint();				
		}					
	}

	public void relaxTotal(){
		tgLayout.relaxTotal();
		repaint();
	}

	private void setUpBackPopup() {
	   backPopup = new PopupMenu();
	   MenuItem menuItem;   
	   menuItem = new MenuItem("auswischen");
	   menuItem.addActionListener(new ActionListener() {
		   public void actionPerformed( ActionEvent e ) {
			   hideAll();
			   clearSelect();
			   repaint();
		   }
	   });
		backPopup.add(menuItem);
		menuItem = new MenuItem("Filter anwenden");
		menuItem.addActionListener(new ActionListener() {
		   public void actionPerformed( ActionEvent e ) {
			   applyFilter();			   
			   resetDamper();
			   repaint();
		   }
	   });	   
	   backPopup.add(menuItem);
	   add(backPopup);
 }
} // end com.touchgraph.graphlayout.TGPanel
