

package org.dronus.graph.gui;

import  java.awt.*;

import org.dronus.graph.Edge;

/**  Edge.
 */
public class LayoutableEdge extends Edge{
	public static Color DEFAULT_COLOR = new Color(100,100,100);
	public static Color DEFAULT_COLOR_AA = new Color(255,255,255,100);
	public static Color defaultColor=DEFAULT_COLOR;
	
	
    public static Color MOUSE_OVER_COLOR = Color.pink;
    public static Font TEXT_FONT = new Font("LucidaBright",Font.PLAIN,10);
	
    public LayoutableNode from,to; //Should be private, changing from effects "from" Node
    
    protected String id = null;
	protected String name = null;
	
  	static public void setAntialias(boolean aa){defaultColor = (aa ? DEFAULT_COLOR_AA : DEFAULT_COLOR  );}
	
	public LayoutableEdge(LayoutableNode f, LayoutableNode t, String type) {
        from = f;
        to = t;
        name=type;
    }
    //public Edge(Node f, Node t) {     this(f, t, DEFAULT_LENGTH);  }

   // setters and getters ...............
   
    public static void setEdgeDefaultColor( Color color ) { DEFAULT_COLOR = color; }
    public static void setEdgeMouseOverColor( Color color ) { MOUSE_OVER_COLOR = color; }
    public String getID()  {     return id;  }
  	public void setID( String id )    {       this.id=id;   }
    public LayoutableNode getOtherEndpt(LayoutableNode n) { //yields false results if Node n is not an endpoint
        if (to != n) return to;
        else return from;
    }
  
    public boolean intersects(Dimension d) {
        int x1 = (int) from.drawx;
        int y1 = (int) from.drawy;
        int x2 = (int) to.drawx;
        int y2 = (int) to.drawy;

        return (((x1>0 || x2>0) && (x1<d.width  || x2<d.width)) &&
                  ((y1>0 || y2>0) && (y1<d.height || y2<d.height) ));

    }

    public double distFromPoint(double px, double py) {
        double x1= from.drawx;
        double y1= from.drawy;
        double x2= to.drawx;
        double y2= to.drawy;

        if (px<Math.min(x1, x2)-8 || px>Math.max(x1, x2)+8 ||
            py<Math.min(y1, y2)-8 || py>Math.max(y1, y2)+8)
            return 1000;

        double dist = 1000;
        if (x1-x2!=0) dist = Math.abs((y2-y1)/(x2-x1)*(px - x1) + (y1 - py));
        if (y1-y2!=0) dist = Math.min(dist, Math.abs((x2-x1)/(y2-y1)*(py - y1) + (x1 - px)));

        return dist;
    }

 
    public static void paintArrow(Graphics g, int x1, int y1, int x2, int y2, Color c, String text,boolean print) {

        double dist=Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
        g.setColor(c);
		
		int dx=(int)(((x1-x2)*3)/dist);
		int dy=(int)(((y1-y2)*3)/dist);
		
		Polygon p= new Polygon();
		p.addPoint(x2,y2);
		p.addPoint(x1+dy,   y1-dx);
		p.addPoint(x1-dy,   y1+dx);
		g.fillPolygon(p);
		g.setColor(print ? Color.DARK_GRAY : Color.LIGHT_GRAY);
		g.setFont(TEXT_FONT);
		if (text!=null) g.drawString(text, (x1+x2)/2, (y1+y2)/2);

    }

    public void paint(Graphics g, boolean selected, boolean print) {
        Color c = selected ? MOUSE_OVER_COLOR : defaultColor;
        

        int x1=(int) from.drawx;
        int y1=(int) from.drawy;
        int x2=(int) to.drawx;
        int y2=(int) to.drawy;
        //if (intersects(tgPanel.getSize())) 
        paintArrow(g, x1, y1, x2, y2, c, name,print);
    }

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

} 