

package org.dronus.graph.gui;

import  java.util.*;

import org.dronus.graph.GraphUtils;
 

public class LocalityUtils {

    GraphPanel tgPanel;
    Locality locality;
    
    public static final int INFINITE_LOCALITY_RADIUS = Integer.MAX_VALUE;
    
    ShiftLocaleThread shiftLocaleThread;
    boolean fastFinishShift=false;  // If finish fast is true, quickly wrap up animation
    
    public LocalityUtils(Locality loc, GraphPanel tgp) {
        locality = loc;
        tgPanel = tgp;
    }

    public void fastFinishAnimation() {
        fastFinishShift = true;
    }

    /** Mark for deletion nodes not contained within distHash. */
    private synchronized boolean markDistantNodes(final Collection subgraph) {
        final boolean[] someNodeWasMarked = new boolean[1]; 
        someNodeWasMarked[0] = false;
        GraphForEachNode fen = new GraphForEachNode() {
            public void forEachNode(LayoutableNode n) {
                if(!subgraph.contains(n)) { 
                    n.markedForRemoval=true;
                    someNodeWasMarked[0] = true;
                }
            }
        };
		
        locality.forAllNodes(fen);
        return someNodeWasMarked[0];
    }

    private synchronized void removeMarkedNodes() {
        final List nodesToRemove = new ArrayList();
        
        GraphForEachNode fen = new GraphForEachNode() {
            public void forEachNode(LayoutableNode n) {
                if(n.markedForRemoval)  {                    
                    nodesToRemove.add(n);
                    n.markedForRemoval=false;    
                }
            }    
        };
	   locality.forAllNodes(fen);
       locality.removeNodes(nodesToRemove);
    }

    /** Add to locale nodes within radius distance of a focal node. */
    private synchronized void addNearNodes(Hashtable distHash, int radius){
        for ( int r=0; r<radius+1; r++ ) {
            Enumeration localNodes = distHash.keys();
            while (localNodes.hasMoreElements()) {
                LayoutableNode n = (LayoutableNode)localNodes.nextElement();
                if(!locality.contains(n) && ((Integer)distHash.get(n)).intValue()<=r) 
					if (tgPanel.filter==null || tgPanel.filter.match(n)){                                        
						n.justMadeLocal = true;
                    	locality.addNodeWithEdges(n);
                    	if (!fastFinishShift && distHash.size()<15) {
                        	try { Thread.sleep(50); } 
                        	catch (InterruptedException ex) {}
                    	}
               		}                
            }
        }
    }

    private synchronized void unmarkNewAdditions() {
        GraphForEachNode fen = new GraphForEachNode() {
            public void forEachNode(LayoutableNode n) {
                n.justMadeLocal=false;
            }    
        };
        locality.forAllNodes(fen);
    }

    /** The thread that gets instantiated for doing the locality shift animation. */
    class ShiftLocaleThread extends Thread {
        Hashtable distHash;
        LayoutableNode focusNode;        
        int radius;
        int maxAddEdgeCount;
        int maxExpandEdgeCount;
        boolean unidirectional;
                
        ShiftLocaleThread(LayoutableNode n, int r, int maec, int meec, boolean unid) {                                         
            focusNode = n;                        
            radius = r;             
            maxAddEdgeCount = maec;
            maxExpandEdgeCount = meec;
            unidirectional = unid;
            start();
         
        }
            
        public void run() {        
            synchronized (LocalityUtils.this) {
            //Make sure node hasn't been deleted
            try{
               if (!locality.getCompleteEltSet().contains(focusNode)) return;                        
               distHash = GraphUtils.calculateDistances(
                          locality.getCompleteEltSet(),focusNode,radius,maxAddEdgeCount,maxExpandEdgeCount,unidirectional,tgPanel.getFilter());                         
               if (markDistantNodes(distHash.keySet())) {
                     for (int i=0;i<5&&!fastFinishShift;i++) {
                         Thread.sleep(100);                      
                     }
                }
                removeMarkedNodes();
                for (int i=0;i<1&&!fastFinishShift;i++) {
                    Thread.sleep(100); 
                }
                addNearNodes(distHash,radius);
                for (int i=0;i<4&&!fastFinishShift;i++) {
                    Thread.sleep(100); 
                }
                unmarkNewAdditions();
               } catch (InterruptedException ex) {}    
                tgPanel.resetDamper();            
            }
        }
    }
            
    public void setLocale(LayoutableNode n, final int radius, final int maxAddEdgeCount, final int maxExpandEdgeCount,
                          final boolean unidirectional) {                                  
        if (radius<0) return;
        if(shiftLocaleThread!=null && shiftLocaleThread.isAlive()) {
            fastFinishShift=true; //This should cause last locale shift to finish quickly
            while(shiftLocaleThread.isAlive())
                try { Thread.sleep(100); } 
                catch (InterruptedException ex) {}
        }
        if (radius == INFINITE_LOCALITY_RADIUS || n==null) {
            tgPanel.filter.applyFilter(locality) ;
            tgPanel.resetDamper();
            return;
        }

        fastFinishShift=false;        
        shiftLocaleThread=new ShiftLocaleThread(n, radius, maxAddEdgeCount, maxExpandEdgeCount, unidirectional);
    }

    public void setLocale(LayoutableNode n, final int radius){                                      
        setLocale(n,radius,1000,1000, false);
    }
    
/*    public synchronized void addAllGraphElts() throws TGException {
        locality.addAll();
		tgPanel.applyFilter();
    }
  */
  
    
   /** Add to locale nodes that are one edge away from a given node.
     * This method does not utilize "fastFinishShift" so it's likely that 
     * synchronization errors will occur.
     */
    public void expandNode(final LayoutableNode n) {
        new Thread() {
            public void run() {
                synchronized (LocalityUtils.this) {
                    if (!locality.getCompleteEltSet().contains(n)) return;                        
                    for(int i=0;i<n.edgeCount();i++) {
                        LayoutableNode newNode = n.edgeAt(i).getOtherEndpt(n);
                        if (!locality.contains(newNode)) {
							if (tgPanel.filter==null || tgPanel.filter.match(newNode))
								continue;
                            newNode.justMadeLocal = true;
                            try {
                                locality.addNodeWithEdges(newNode);
                                Thread.sleep(50); 
                            } catch ( InterruptedException ex ) {}         
                        }
                        else if (!locality.contains(n.edgeAt(i))) {
                            locality.addEdge(n.edgeAt(i));
                        }
                    }
                    try { Thread.sleep(200); } 
                    catch (InterruptedException ex) {}         
                    unmarkNewAdditions();
                    tgPanel.resetDamper();
                }
            }
        }.start();
    }
    
   /** Hides a node, and all the nodes attached to it. */
    public synchronized void hideNode( final LayoutableNode hideNode ) {
        if (hideNode==null) return;
        new Thread() {
            public void run() {
                synchronized(LocalityUtils.this) {
                    if (!locality.getCompleteEltSet().contains(hideNode)) return;                                                               
                    
                    locality.removeNode(hideNode); //Necessary so that node is ignored in distances calculation.
                    if (hideNode==tgPanel.getSelect()) {
                        tgPanel.clearSelect();
                    }

                    Collection subgraph = GraphUtils.getLargestConnectedSubgraph(locality);
                    markDistantNodes(subgraph);                      
                    tgPanel.repaint();
                    try { Thread.sleep(200); } 
                    catch (InterruptedException ex) {}         
                    removeMarkedNodes();                    
                    
                    tgPanel.resetDamper();
                }
            }
        }.start();
    }

   /** Opposite of expand node, works like hide node except that the selected node is not hidden.*/
    public synchronized void collapseNode( final LayoutableNode collapseNode ) {
        if (collapseNode==null) return;
        new Thread() {
            public void run() {
                synchronized(LocalityUtils.this) {
                    if (!locality.getCompleteEltSet().contains(collapseNode)) return;                                                               
                    
                    locality.removeNode(collapseNode); //Necessary so that node is ignored in distances calculation.                                                               
                    Collection subgraph = GraphUtils.getLargestConnectedSubgraph(locality);
                    markDistantNodes(subgraph);                                          
                    locality.addNodeWithEdges(collapseNode); // Add the collapsed node back in.
                    tgPanel.repaint();
                    try { Thread.sleep(200); } 
                    catch (InterruptedException ex) {}         
                    removeMarkedNodes();                    
                    
                    tgPanel.resetDamper();
                }
            }
        }.start();
    }
} // end com.touchgraph.graphlayout.LocalityUtils
