
package org.dronus.graph.gui;

import  org.dronus.graph.gui.interact.*;
import org.dronus.graph.nodefilter.*;
import org.dronus.hansi.Happ;

import  java.awt.*;
import  java.awt.event.*;

public class MutaGUIPanel extends Panel {
    public PopupMenu glPopup;
    protected MutaGraphPanel tgPanel= new MutaGraphPanel();
    public ZoomScroll zoomScroll = new ZoomScroll(tgPanel);
	public LocalityScroll localityScroll = new LocalityScroll(tgPanel);
    
    GraphNavigateUI navUI;
	GraphEditUI editUI;    
       
	String url="";       
      
	public MutaGUIPanel(String _url) {
		url=_url;
	    buildPanel();

		tgPanel.setFilter(new BasicNodeFilter(true,false,false));
		tgPanel.tgLensSet.addLens(zoomScroll.getLens());
		tgPanel.tgLensSet.addLens(tgPanel.adjustOriginLens);
		navUI=new GraphNavigateUI(this);		
		editUI=new GraphEditUI(this);
		navUI.activate();
        setVisible(true);        		
		requestFocus();
		tgPanel.addKeyListener(new KeyAdapter(){
			 public void keyReleased(KeyEvent e) {			 	
				switch (e.getKeyCode()){
					case KeyEvent.VK_Z: tgPanel.centerNode(tgPanel.getSelect());break;
					case KeyEvent.VK_F:
						if (tgPanel.getSelect()!=null){
							 tgPanel.getSelect().fixed=!tgPanel.getSelect().fixed;
							 tgPanel.repaint();
						}						
						break;
					case KeyEvent.VK_S:
						
					case KeyEvent.VK_DELETE:
						LayoutableNode sel; 
						if((sel=tgPanel.getSelect())!=null){ 
							tgPanel.clearSelect();
						//	tgPanel.visibleLocality.removeNode(sel); 							
						}
						break;
					case KeyEvent.VK_I:
						System.out.println(tgPanel.getVisibleLocality().nodeCount()+" Nodes, "+tgPanel.getVisibleLocality().edgeCount()+" Edges.");
						break;					
				}
			  }
		 });
    }
	public MutaGraphPanel getTGPanel() { 	return tgPanel;  }
    public LocalityScroll getLocalityScroll()    {      return localityScroll;   }

	public void buildMenu(MenuBar mb){
		Menu menu=new Menu("Graph");
		MenuItem m;		
		
		m = new MenuItem("Layout speichern...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tgPanel.saveLayout();
			}
		});		
		m.setShortcut(new MenuShortcut(KeyEvent.VK_S));
		menu.add(m);
		
		m = new MenuItem("Layout laden...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tgPanel.loadLayout();
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_L));		
		menu.add(m);
		
		menu.addSeparator();
		m = new MenuItem("Graph drucken...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new PrintFrame(tgPanel,false);
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_P));		
		menu.add(m);
		
		m = new MenuItem("Graph postscripten...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new PrintFrame(tgPanel,true);				
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_P,true));		
		menu.add(m);
		
		menu.addSeparator();		
		m = new MenuItem("Info ausgeben");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println(tgPanel.getVisibleLocality().nodeCount()+" Nodes, "+tgPanel.getVisibleLocality().edgeCount()+" Edges.");			
			}
		});	
		m.setShortcut(new MenuShortcut(KeyEvent.VK_I));
		menu.add(m);
	
		mb.add(menu);

		menu=new Menu("Suche");
		m = new MenuItem("sichtbaren Knoten suchen...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				searchDialog(false);
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_F));		
		menu.add(m);
		m = new MenuItem("Knoten suchen...");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				searchDialog(true);				
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_F,true));		
		menu.add(m);
		
		m = new MenuItem("Orte in Hamburg");
		m.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new Happ(tgPanel);				
			}
		});
		m.setShortcut(new MenuShortcut(KeyEvent.VK_F,true));		
		menu.add(m);
		
		mb.add(menu);
	}
	
	void searchDialog(boolean extended){
		NodeSearchDialog nsd;
		if (extended)
			nsd = new NodeSearchDialog(tgPanel, tgPanel.completeEltSet,true);
		else 
			nsd = new NodeSearchDialog(tgPanel, tgPanel.visibleLocality,false);
		LayoutableNode[] nodes=nsd.getResults();
		if (nodes==null || nodes.length==0) return;
		for (int i=0; i<nodes.length; i++){
			tgPanel.visibleLocality.addNodeWithEdges(nodes[i]);					
		}							
		tgPanel.centerNode(nodes[0]);
		tgPanel.setSelect(nodes[0]);
	}
	
    public void buildPanel() {
		Panel p;
		
		
		final Panel topPanel = new Panel();		
		topPanel.setLayout(new BorderLayout());
		final Panel controlPanel = new Panel();
		controlPanel.setLayout(new GridLayout());
		final Panel bottomPanel = new Panel();
		        
		setLayout(new BorderLayout());
		add(topPanel, BorderLayout.NORTH);
       
        //final TextField urlField= new TextField("http://www.bolau.de/");
		final TextField urlField= new TextField(url);
		final TextField setField= new TextField("");
		setField.setColumns(10);
		final TextField queryField= new TextField("");
		queryField.setColumns(20);
		ActionListener reload=new ActionListener(){
			public void actionPerformed(ActionEvent e){
				tgPanel.setUrl(urlField.getText(),queryField.getText(),setField.getText());					
				requestFocus();
			}; 
		};
		
		urlField.addActionListener(reload);
		setField.addActionListener(reload);
		queryField.addActionListener(reload);		
		
		
		controlPanel.add(p=new Panel());
		p.setLayout(new BorderLayout());		
		p.add(new Label("URL:"),BorderLayout.WEST);
		p.add(urlField);
		controlPanel.add(p=new Panel());
		p.setLayout(new BorderLayout());
		p.add(new Label("Set:"),BorderLayout.WEST);
		p.add(setField);
		controlPanel.add(p=new Panel());
		p.setLayout(new BorderLayout());
		p.add(new Label("Parameter:"),BorderLayout.WEST);
		p.add(queryField);
		
		topPanel.add(controlPanel,BorderLayout.NORTH);
		topPanel.add(localityScroll.getLocalitySB(),BorderLayout.WEST);		
		topPanel.add(zoomScroll.getZoomSB(),BorderLayout.CENTER);
		topPanel.add(p=new Panel(),BorderLayout.EAST);				
		FlowLayout lo=new FlowLayout();
		lo.setHgap(0);lo.setVgap(0);
		p.setLayout(lo);
		final Checkbox checkAA=new Checkbox("AA");
		final Checkbox checkEdit=new Checkbox("Edit");		
		final Checkbox checkStandard=new Checkbox("Alle"); checkStandard.setState(true); 
		final Checkbox checkSet=new Checkbox("Sets"); 
		final Checkbox checkType= new Checkbox("Typen");						
		final Checkbox checkMotion=new Checkbox("Motion"); checkMotion.setState(true);
		Button buttonRelax=new Button("Relax!");			
		p.add(checkAA);		
		p.add(checkStandard);
		p.add(checkSet);
		p.add(checkType);
		p.add(checkEdit);
		p.add(checkMotion);
		p.add(buttonRelax);
		
		buttonRelax.addActionListener (new ActionListener(){
			public void actionPerformed(ActionEvent e){
				tgPanel.relaxTotal();
			}; 
		});
				
		ItemListener filterChanged=new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				tgPanel.setFilter(new BasicNodeFilter(checkStandard.getState(),checkSet.getState(),checkType.getState() ));	
			}
		};

		checkAA.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				tgPanel.setAntialias(checkAA.getState());				
			}
		});		
		checkStandard.addItemListener(filterChanged);
		checkSet.addItemListener(filterChanged);
		checkType.addItemListener(filterChanged);
		checkEdit.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				if (checkEdit.getState()){
					navUI.deactivate(); editUI.activate();
					add(bottomPanel, BorderLayout.SOUTH);
					validate();
				}else{
					navUI.activate(); editUI.deactivate();
					remove(bottomPanel);
					validate();
				}				
			}
		});
		checkMotion.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e){
				if (checkMotion.getState()){
					tgPanel.resetDamper();
					tgPanel.start();					
				}else{
					tgPanel.stop();
				}				
			}
		});				
				
		
		
		final TextField nodeTypeField=new TextField(tgPanel.nodeType);
		nodeTypeField.addTextListener(new TextListener(){
			public void textValueChanged(TextEvent e){tgPanel.nodeType=nodeTypeField.getText();}
		});
		final TextField edgeTypeField=new TextField(tgPanel.edgeType);
		edgeTypeField.addTextListener(new TextListener(){
			public void textValueChanged(TextEvent e){
				tgPanel.edgeType=edgeTypeField.getText();
			};
		});
		
		bottomPanel.add(new Label("Knoten-Typ:"));
		bottomPanel.add(nodeTypeField);
		bottomPanel.add(new Label("Kanten-Typ:"));
		bottomPanel.add(edgeTypeField);		
        
        //add(tgPanel,BorderLayout.SOUTH);
		Panel scrollPanel = new Panel();
		scrollPanel.setLayout(new BorderLayout());      	
        scrollPanel.add(tgPanel);
      //  Dimension riesig=new Dimension(400,400);
      // tgPanel.setSize(riesig);		
        add(scrollPanel,BorderLayout.CENTER);
    }

} // end com.touchgraph.graphlayout.GLPanel
