
package org.dronus.graph.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import org.dronus.dope.FilePool;
import org.dronus.dope.Hasher;


public class MutaGraphPanel extends GraphPanel {
	String url,query;
	String setid="";
	public String nodeType="Person", nodeName="irgendwer", edgeType="ist";
	
	static public String enc(String s){
		return URLEncoder.encode(s);
	}
	public void setSet(String _set){
		setUrl(url,query,_set);
	} 
	public void setUrl(String _url, String _query, String set){
		clearSelect();
		clearAll();
		stop();
		url=_url;
		query=_query;
		if (query==null) query="";
		if (set==null) set="";
		try {
			System.out.print("MutaGraphPanel: Reading graph, ");			
			setid=performServerFunction("?action=getset&query="+enc(query)+"&set="+enc(set));
			System.out.println("	connected to "+url);			
			BufferedReader r=getServerReader("?action=getnodes&query="+enc(query)+"&setid="+setid);			
			String line;
			while((line=r.readLine())!=null)	{
				StringTokenizer tok=new StringTokenizer(line,";");
				String id=tok.nextToken(), name=tok.nextToken(), type=tok.nextToken();	
				super.addNode(new LayoutableNode(id, name, type));
			}
			System.out.print(getVisibleLocality().nodeCount()+" Nodes; ");			
			r= new BufferedReader (new InputStreamReader(new URL(url+"?action=getedges&setid="+setid).openStream()));
			while((line=r.readLine())!=null)	{
				StringTokenizer tok=new StringTokenizer(line,";");
				String id=tok.nextToken(), n1=tok.nextToken(), n2=tok.nextToken();
				String name=tok.nextToken();				
				LayoutableNode node1=findNode(n1), node2=findNode(n2);
				LayoutableEdge e=new LayoutableEdge(node1,node2, null);
				e.setID(id); e.setName(name);
				if (node1!=null && node2!=null) super.addEdge(e);
				else
					System.out.println("\n Oha! Edge in ass: "+id+" from "+n1+" to "+n2);
			}
			System.out.println(getVisibleLocality().edgeCount()+" Edges. Loading complete.");			
			applyFilter();
			start();								
		} catch (Exception e) {
			if (e instanceof MalformedURLException) 
				System.out.println("bad URL:"+url);
			else if (e instanceof IOException)
				System.out.println("connection lost:"+url);
			else
				e.printStackTrace();
		}		
	}
		
	public void addEdge(LayoutableEdge e) {
		String type;
		if (e.getName()==null)  type=edgeType;
		else type=e.getName();
		String id = performServerFunction("?action=addedge&n1="+enc(e.from.getID())+"&n2="+enc(e.to.getID())+"&type="+enc(type));
		if(id != null){
			e.setID(id);
			e.setName(type);  				
			super.addEdge(e);
		}
	}

	public void deleteEdge(LayoutableEdge edge) {
		String id = edge.getID();
		if (performServerAction("?action=deleteedge&id="+enc(id)))
			super.deleteEdge(edge);		
	}
		
	public boolean eraseNode(LayoutableNode node) {		
		if (performServerAction("?action=deletenode&id="+enc(node.getID())))
			return super.deleteNode(node);
		return false;		
	}




	class NameDialog extends Dialog{
		TextField namef;
		TextField typef;
		NameDialog(Frame f, String name, String type){
			super(f, "Knoteneigenschaften",true);
			setLocationRelativeTo(f);
			Panel p =new Panel();
			add(p);			
			
			KeyListener kl=new KeyAdapter(){
					public void keyReleased(KeyEvent e) {
						if (e.getKeyCode()==KeyEvent.VK_ESCAPE) dispose();
					}
			};
			ActionListener al= new ActionListener(){
				public void actionPerformed(ActionEvent e){						
					setVisible(false);						
				}					
			};
			namef= new TextField(name);
			namef.setColumns(30);					
			namef.addActionListener(al);
			namef.addKeyListener(kl);			
			p.add(new Label("Name:"));
			p.add(namef);
			typef= new TextField(type);			
			typef.setColumns(30);					
			typef.addActionListener(al);
			typef.addKeyListener(kl);
			p.add(new Label("Typ:"));			
			p.add(typef);
																	
			pack();
			show();
		}
		public String getResultName(){return namef.getText();}
		public String getResultType(){return typef.getText();}
	}

	Frame getFrame(){
		Component c=this;
		while ( c!= null && ! (c instanceof Frame ) ) c=c.getParent();
		return (Frame)c;
	}

	public synchronized LayoutableNode addNode(String type, String name){					
		String result=performServerFunction("?action=addnode&name="+enc(name)+"&type="+enc(type)+"&setid="+enc(setid));
		LayoutableNode n= findNode(result);
		if (n==null){
			n= new LayoutableNode(result, name, type);
			 //if (n instanceof TypeNode)	((TypeNode)n).update(visibleLocality);
		}
		super.addNode(n);
		return n;
	}

	public synchronized void changeNode(LayoutableNode n){
		NameDialog nd=new NameDialog(getFrame(),n.getLabel(),n.getType());
		String name=nd.getResultName();
		String type=nd.getResultType();
		String result=performServerFunction("?action=editnode&id="+enc(n.getID())+"&name="+enc(name)+"&type="+enc(type));
		if (result.equals("false")) return; 
		n.setLabel(name); n.setType(type);
	}

	public synchronized LayoutableNode addNode(){								
		NameDialog nd=new NameDialog(getFrame(),"",nodeType);
		String type=nd.getResultType();	
		String	name=nd.getResultName();
		if (name.equals("")) return null;
		if(type.equals("Datei")){			
			try {
				Hasher h = new Hasher(new URL(name));
				LayoutableNode source = addNode("Quelle",name);
				LayoutableNode file=addNode(type,h.getHash());
				LayoutableEdge e= new LayoutableEdge(source,file,"Quelle");				
				addEdge(e);	
				return (file);
			} catch (Exception e) {
				System.out.println("URL not found:"+name);
				return null;
			}								
		} else		
			return addNode(type,name);
	}
	public void explodeNode(final LayoutableNode n){								
		(new Thread(){
			public void run() {
				setPriority(Thread.MIN_PRIORITY);
				explodeNodeRun(n);
			}
		}).start();			
	}
	void explodeNodeRun(LayoutableNode n){
		Set s=FilePool.getDefaultPool().getBySomething(n.getLabel(),n.getType());
		Iterator it=s.iterator();
		while (it.hasNext()){
			LayoutableNode fn=new LayoutableNode(null,((File)it.next()).getAbsolutePath(), LayoutableNode.NT_FILE);
			
			super.addNode(fn);
			super.addEdge(new LayoutableEdge(n,fn,""));
			fn.setLocation(n.getLocation());			
		}
	};	
	
	BufferedReader getServerReader(String s)throws Exception{
		return new BufferedReader (new InputStreamReader( (new URL(url+s)).openStream() ) );
	}
	boolean performServerAction(String s){
		return ((new Boolean(performServerFunction(s))).booleanValue() );
	}	
	String performServerFunction(String s){		
		try {
			URL u = new URL(url+s);
			System.out.print("ServerFunction "+s);
			BufferedReader r;
			r = new BufferedReader(new InputStreamReader(u.openStream()));
			String line=r.readLine();
			System.out.println(" : "+line);
			return line;
		} catch (IOException e) {			
			System.out.println("ServerFunction no connect: "+url);		
		}
		return null;					
	}
	public void loadLayout(){
		//URL u=new URL("file://test.mgl");
		FileDialog fd = new FileDialog(getFrame());
		fd.setDirectory("layouts");
		fd.show();			
		File f=new File(fd.getDirectory()+fd.getFile());
		visibleLocality.removeAll();
		try {
			BufferedReader s = new BufferedReader(new FileReader(f));
			String l;
			while((l=s.readLine())!=null){
				StringTokenizer tok = new StringTokenizer(l);
				String id = tok.nextToken();
				LayoutableNode n=completeEltSet.findNode(id);
				if (n==null) continue;
				visibleLocality.addNodeWithEdges(n);
				n.x=(new Double(tok.nextToken())).doubleValue();
				n.y=(new Double(tok.nextToken())).doubleValue();
				n.fixed = ((new Boolean(tok.nextToken())).booleanValue());
				n.stop();	
			}			
			updateDrawPositions();
		} catch (Exception e) {
			if (e instanceof FileNotFoundException) System.out.println("file not found:"+f.toString());  
			else System.out.println("not a valid layout-file.");
			e.printStackTrace();
		}						
	}

	public void saveLayout(){
		try {
			FileDialog fd = new FileDialog(getFrame());
			fd.setDirectory("layouts");
			fd.show();			
			File f=new File(fd.getDirectory()+fd.getFile());			
			//URL u=new URL("file://test.mgl");
			final PrintWriter s = new PrintWriter(new FileWriter(f));
			GraphForEachNode saver=new GraphForEachNode(){
					public void forEachNode(LayoutableNode n) {
						s.println(n.getID()+" "+n.x+" "+n.y+" "+n.fixed);
						System.out.println(n.getID()+" "+n.x+" "+n.y+" "+n.fixed);
					} 
			};
			visibleLocality.forAllNodes(saver);
			s.close();			
		} catch (IOException e) {
			System.out.print(e);
		}	
	}
	 		
}
