

package org.dronus.graph.gui.interact;
import org.dronus.graph.gui.*;


import  java.awt.*;
import  java.awt.event.*;

/**  DragAddUI contains code for adding nodes + edges by dragging.
  *   
  * @author   Alexander Shapiro                                        
  * @version  1.21  $Id: DragAddUI.java,v 1.12 2002/04/01 05:53:49 x_ander Exp $
  */
public class DragAddUI extends GraphAbstractDragUI implements GraphPaintListener {

    Point mousePos = null;
	Point startPos = null;
    LayoutableNode dragAddNode = null;

	
  // ............

   /** Constructor with provided TGPanel <tt>tgp</tt>.
     */ 
    public DragAddUI( GraphPanel tgp ) {
        super(tgp); 
    }

    public void preActivate() {
        mousePos=null;
        tgPanel.addPaintListener(this);
    }

    public void preDeactivate() {
        tgPanel.removePaintListener(this);
    };

    public void mousePressed( MouseEvent e ) {
        dragAddNode = tgPanel.getMouseOverN();
        startPos=e.getPoint();
    }    

    public synchronized void mouseReleased( MouseEvent e ) {
        LayoutableNode mouseOverN = tgPanel.getMouseOverN();

        if (mouseOverN!=null && dragAddNode!=null && mouseOverN!=dragAddNode) {
            LayoutableEdge ed=tgPanel.findEdge(dragAddNode,mouseOverN);
            if (ed==null) tgPanel.addEdge(new LayoutableEdge(dragAddNode,mouseOverN, null)); 
            else tgPanel.deleteEdge(ed);
        } else if (mouseOverN!=null && dragAddNode==null ) {
			LayoutableNode n =tgPanel.addNode(); 
			tgPanel.addEdge(new LayoutableEdge(n, mouseOverN, null));
			n.drawx = startPos.x; 
			n.drawy = startPos.y;             
			tgPanel.updatePosFromDraw(n); 
				tgPanel.repaint();
        } else if ( mouseOverN == null && dragAddNode != null ) {
            LayoutableNode n =tgPanel.addNode(); 
            tgPanel.addEdge(new LayoutableEdge(dragAddNode,n, null));
			n.drawx = mousePos.x; 
			n.drawy = mousePos.y;			 
            tgPanel.updatePosFromDraw(n);
			tgPanel.repaint();
        } else if (!mouseWasDragged){
       		LayoutableNode n =tgPanel.addNode();
			if (n!=null){
				n.drawx = tgPanel.getMousePos().x;
				n.drawy = tgPanel.getMousePos().y;
				tgPanel.updatePosFromDraw(n);
				tgPanel.repaint();
    		} 
	     }
        
        if (mouseWasDragged) { //Don't reset the damper on a mouseClicked
            tgPanel.resetDamper();            
        }   

        dragAddNode=null;
    }

    public void mouseDragged(MouseEvent e) {    
        mousePos=e.getPoint();
        tgPanel.repaint();
    }

    public void paintFirst(Graphics g) {};
    public void paintLast(Graphics g) {};

    public void paintAfterEdges(Graphics g) {

        if(mousePos==null) return;
        LayoutableNode mouseOverN = tgPanel.getMouseOverN();
		Color c;
        if (mouseOverN==null) {
            g.setColor(LayoutableNode.DEFAULT_COLOR);
            g.drawRect(mousePos.x-7, mousePos.y-7, 14, 14);
			c=LayoutableEdge.MOUSE_OVER_COLOR;                        
        }else c=LayoutableEdge.DEFAULT_COLOR;
        
        if (dragAddNode!=null) 
        	LayoutableEdge.paintArrow(g, (int) dragAddNode.drawx, (int) dragAddNode.drawy,
               mousePos.x, mousePos.y, c, null, false);
        else if (startPos!=null)
		LayoutableEdge.paintArrow(g, (int) startPos.x, (int) startPos.y,
					   mousePos.x, mousePos.y, c, null, false);
    }

} // end com.touchgraph.graphlayout.interaction.DragAddUI
