

package org.dronus.graph.gui.interact;

import org.dronus.graph.gui.*;

import  java.awt.event.*;

/** TGAbstractDragUI allows one to write user interfaces that handle
  * what happends when a mouse is pressed, dragged, and released.
  *
  * @author   Alexander Shapiro
  * @version  1.21  $Id: TGAbstractDragUI.java,v 1.11 2002/04/01 05:52:38 x_ander Exp $
  */
public abstract class GraphAbstractDragUI extends GraphUserInterface {

    public GraphPanel tgPanel;

    private ADUIMouseListener ml;
    private ADUIMouseMotionListener mml;
    public boolean mouseWasDragged; //To differentiate between mouse pressed+dragged, and mouseClicked

  // ............

   /** Constructor with TGPanel <tt>tgp</tt>.
     */
    public GraphAbstractDragUI(GraphPanel tgp) {
        tgPanel=tgp;
        ml =new ADUIMouseListener();
        mml=new ADUIMouseMotionListener();
    }

     public final void activate() {
        preActivate();
        tgPanel.addMouseListener(ml);
        tgPanel.addMouseMotionListener(mml);
        mouseWasDragged=false;
     }

    public final void activate(MouseEvent e) {
        activate();
        mousePressed(e);
    }

    public final void deactivate() {
        preDeactivate();
        tgPanel.removeMouseListener(ml);
        tgPanel.removeMouseMotionListener(mml);
        super.deactivate(); //To activate parentUI from TGUserInterface
    }

    public abstract void preActivate();
    public abstract void preDeactivate();

    public abstract void mousePressed( MouseEvent e );
    public abstract void mouseDragged( MouseEvent e );
    public abstract void mouseReleased( MouseEvent e );


    private class ADUIMouseListener extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            GraphAbstractDragUI.this.mousePressed(e);
        }

        public void mouseReleased(MouseEvent e) {
            GraphAbstractDragUI.this.mouseReleased(e);
            if (selfDeactivate) deactivate();
        }
    }

    private class ADUIMouseMotionListener extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent e) {
            mouseWasDragged=true;
              GraphAbstractDragUI.this.mouseDragged(e);
        }
    }

} // end com.touchgraph.graphlayout.interaction.TGAbstractDragUI
