
package org.dronus.graph.gui.interact;

import  org.dronus.graph.*;
import org.dronus.graph.gui.*;

import  java.awt.*;
import java.awt.datatransfer.StringSelection;
import  java.awt.event.*;

public class GraphNavigateUI extends GraphUserInterface {
    MutaGraphPanel tgPanel;    
    
    GLNavigateMouseListener ml;
        
    GraphAbstractDragUI hvDragUI;
    GraphAbstractDragUI rotateDragUI;
    DragNodeUI dragNodeUI;
    LocalityScroll localityScroll;
    PopupMenu nodePopup;    
    PopupMenu edgePopup;    
    LayoutableNode popupNode;
    LayoutableEdge popupEdge;
    
    public GraphNavigateUI( MutaGUIPanel glp) {
        this(glp.getTGPanel());
        localityScroll = glp.getLocalityScroll();        
    }
	public GraphNavigateUI( MutaGraphPanel glp ) {
		tgPanel = glp;                
		localityScroll = new LocalityScroll(glp);
		hvDragUI = glp.getHVScroll().getHVDragUI();
		dragNodeUI = new DragNodeUI(tgPanel);                    
		ml = new GLNavigateMouseListener();    
		setUpNodePopup();
		setUpEdgePopup();        
		tgPanel.add(nodePopup);
		tgPanel.add(edgePopup);
	}
    
    
    public synchronized void activate() {        
        if (active) return;
        active=true;
        tgPanel.addMouseListener(ml);
    }
    
    public synchronized void deactivate() {
    	if (!active) return;
    	active=false;
        tgPanel.removeMouseListener(ml);
    }
    
    class GLNavigateMouseListener extends MouseAdapter {
    
        public void mousePressed(MouseEvent e) {
            LayoutableNode mouseOverN = tgPanel.getMouseOverN();
            
            if (e.getModifiers() == MouseEvent.BUTTON1_MASK) { 
                if (mouseOverN == null) 
                    hvDragUI.activate(e);
                else 
                    dragNodeUI.activate(e);                    
            }
        }
		
        public void mouseClicked(MouseEvent e) {
            LayoutableNode mouseOverN = tgPanel.getMouseOverN();
            if (e.getModifiers() == MouseEvent.BUTTON1_MASK) {                 
                if ( mouseOverN != null) {                        
					if (e.getClickCount()==1){					
                        tgPanel.setSelect(mouseOverN);
               			if (localityScroll.getLocalityRadius()!=LocalityUtils.INFINITE_LOCALITY_RADIUS)
                            	tgPanel.setLocale(mouseOverN, localityScroll.getLocalityRadius());                        
                			}else{ //double click
						mouseOverN.defaultAction();						 
                	}
                }
            }    
        }    
        
        public void mouseReleased(MouseEvent e) {
            if (e.isPopupTrigger()) {
                popupNode = tgPanel.getMouseOverN();
                popupEdge = tgPanel.getMouseOverE();
                if (popupNode!=null) {
                    tgPanel.setMaintainMouseOver(true);
                    ((RollbackPopupMenu)nodePopup).mark(); 
					if (popupNode instanceof PopupNode){						
						tgPanel.remove(nodePopup);
						((PopupNode)popupNode).extendNodePopup(nodePopup, tgPanel);
						tgPanel.invalidate();
						tgPanel.add(nodePopup);						
						tgPanel.validate();
					}                      
                    nodePopup.show(tgPanel, e.getX(), e.getY());
					tgPanel.setMaintainMouseOver(false);
					tgPanel.setMouseOverN(null);
					tgPanel.repaint();
					((RollbackPopupMenu)nodePopup).rollback();
										
                }
                else if (popupEdge!=null) {
                    tgPanel.setMaintainMouseOver(true);
                    edgePopup.show(e.getComponent(), e.getX(), e.getY());
					tgPanel.setMaintainMouseOver(false);
					tgPanel.setMouseOverE(null);					
                } else tgPanel.backPopup.show(e.getComponent(), e.getX(), e.getY());
            }
        }    

    }
    
    public RollbackPopupMenu navigateMenu(){
		RollbackPopupMenu nodePopup = new RollbackPopupMenu("Knoten");
		MenuItem menuItem;
		
		menuItem = new MenuItem("Kopieren");
		ActionListener copyAction = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if(popupNode!=null) Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection((popupNode.getLabel()+" ("+popupNode.getType()+")").toString()),null);					
				}
			};

		menuItem.addActionListener(copyAction);
		nodePopup.add(menuItem);
		
		menuItem = new MenuItem("Explodieren");
		ActionListener expandAction = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if(popupNode!=null) tgPanel.explodeNode(popupNode);
					//if(popupNode!=null) tgPanel.expandNode(popupNode);
				}
			};

		menuItem.addActionListener(expandAction);
		nodePopup.add(menuItem);

		menuItem = new MenuItem("Einsammeln");
		ActionListener collapseAction = new ActionListener() {
				public void actionPerformed(ActionEvent e) {                    
					if(popupNode!=null) tgPanel.collapseNode(popupNode );
				}
			};

		menuItem.addActionListener(collapseAction);
		nodePopup.add(menuItem);    
		return nodePopup; 
    }

	private void setUpNodePopup() {
		nodePopup=navigateMenu();
	}

    private void setUpEdgePopup() {        
        edgePopup = new PopupMenu();
        MenuItem menuItem;
                 
        menuItem = new MenuItem("Kante verstecken");
        ActionListener hideAction = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if(popupEdge!=null) {
                        tgPanel.hideEdge(popupEdge);
                    }
                }
            };
                    menuItem.addActionListener(hideAction);
        edgePopup.add(menuItem);        
    }
} 
