
package org.dronus.graph.gui.interact;

import org.dronus.graph.gui.*;
import  java.awt.event.*;


public class HVScroll  {

    HVLens hvLens;
    HVDragUI hvDragUI;
    HVScrollToCenterUI hvScrollToCenterUI;
    public boolean scrolling;

  
    private GraphPanel tgPanel;
    private LensSet tgLensSet;

    Point2D offset;

    public HVScroll(GraphPanel tgp, LensSet tgls) {
        tgPanel=tgp;
        tgLensSet=tgls;

        offset=new Point2D(0,0);
        scrolling = false;
  
        hvLens=new HVLens();
        hvDragUI = new HVDragUI(); //Hopefully this approach won't eat too much memory
        hvScrollToCenterUI = new HVScrollToCenterUI();
        
    }
    public HVDragUI getHVDragUI() { return hvDragUI; }

    public HVLens getLens() { return hvLens; }

    public GraphAbstractClickUI getHVScrollToCenterUI() { return hvScrollToCenterUI; }

    public Point2D getDrawCenter() { //Should probably be called from tgPanel
        return new Point2D(tgPanel.getSize().width/2,tgPanel.getSize().height/2);
    }
     class HVLens extends AbstractLens {
        protected void applyLens(Point2D p) {            
            p.x=p.x-offset.x;
            p.y=p.y-offset.y;
        }

        protected void undoLens(Point2D p) {
            p.x=p.x+offset.x;
            p.y=p.y+offset.y;
        }
     }

     public void scrollAtoB(Point2D drawFrom, Point2D drawTo) {
        Point2D from = tgLensSet.convDrawToReal(drawFrom.x,drawFrom.y);
        Point2D to = tgLensSet.convDrawToReal(drawTo.x,drawTo.y);
        offset.x=(offset.x+(from.x-to.x));
        offset.y=(offset.y+(from.y-to.y));
     }


     public void slowScrollToCenter(final LayoutableNode n) {
         final Point2D drawTo = getDrawCenter();
         scrolling = true;
         Thread scrollThread = new Thread() {
             public void run() {
                for(int i=0;i< 10;i++) {
           			double dx = (drawTo.x-n.drawx)/2.0;
					double dy = (drawTo.y-n.drawy)/2.0; 
					scrollAtoB(new Point2D(0,0),new Point2D(dx,dy)); //for good measure
                    tgPanel.repaintAfterMove();
                    try {
                           Thread.sleep(50);
                    } catch (InterruptedException ex) {    }
                }
                scrollAtoB(new Point2D(n.drawx,n.drawy),getDrawCenter()); //for good measure
                tgPanel.repaintAfterMove();
                HVScroll.this.scrolling = false;
            }
        };
        scrollThread.start();
     }

    class HVScrollToCenterUI extends GraphAbstractClickUI {
        public void mouseClicked(MouseEvent e) {
            LayoutableNode mouseOverN=tgPanel.getMouseOverN();
            if(!scrolling && mouseOverN!=null)
                slowScrollToCenter(mouseOverN);
        }
    }

    class HVDragUI extends GraphAbstractDragUI{
        Point2D lastMousePos;
        HVDragUI() { super(HVScroll.this.tgPanel); }

        public void preActivate() {}
        public void preDeactivate() {}

        public void mousePressed(MouseEvent e) {
            lastMousePos = new Point2D(e.getX(), e.getY());
        }
        public void mouseReleased(MouseEvent e) {}
        public void mouseDragged(MouseEvent e) {
            if(!scrolling) scrollAtoB(lastMousePos, new Point2D(e.getX(), e.getY()));
            lastMousePos.x=e.getX();lastMousePos.y=e.getY();
            this.tgPanel.repaintAfterMove();
        }
    }

} // end com.touchgraph.graphlayout.interaction.HVScroll
